// Demonstrate Lighting

#include "Angel.h"
#include "uofrGraphics.h"


//Define some geometry and colours here
typedef Angel::vec4 point4;

point4 cubeVerts[] = 
{
	point4( 0.5, 0.5, 0.5, 1), //0
	point4( 0.5, 0.5,-0.5, 1), //1
	point4( 0.5,-0.5, 0.5, 1), //2
	point4( 0.5,-0.5,-0.5, 1), //3
	point4(-0.5, 0.5, 0.5, 1), //4
	point4(-0.5, 0.5,-0.5, 1), //5
	point4(-0.5,-0.5, 0.5, 1), //6
	point4(-0.5,-0.5,-0.5, 1)  //7
};

point4 points[] = 
{
#define wireCubeStart 0
#define wireCubeVertices 30
	//Wire Cube - 30 vertices := 6 faces * (4 vertices per face + 1 to close loop)
	cubeVerts[0], cubeVerts[4], cubeVerts[6], cubeVerts[2], cubeVerts[0], //front
	cubeVerts[1], cubeVerts[0], cubeVerts[2], cubeVerts[3], cubeVerts[1], //right
	cubeVerts[5], cubeVerts[1], cubeVerts[3], cubeVerts[7], cubeVerts[5], //back 
	cubeVerts[4], cubeVerts[5], cubeVerts[7], cubeVerts[6], cubeVerts[4], //left 
	cubeVerts[4], cubeVerts[0], cubeVerts[1], cubeVerts[5], cubeVerts[4], //top
	cubeVerts[6], cubeVerts[7], cubeVerts[3], cubeVerts[2], cubeVerts[6], //bottom

#define solidCubeStart 30
#define solidCubeVertices 36
    //Solid Cube - 36 vertices := 6 faces * 2 Triangles per face * 3 vertices per triangle
	cubeVerts[0], cubeVerts[4], cubeVerts[6],  //front
	cubeVerts[0], cubeVerts[6], cubeVerts[2],  
	cubeVerts[1], cubeVerts[0], cubeVerts[2],  //right 
	cubeVerts[1], cubeVerts[2], cubeVerts[3],
	cubeVerts[5], cubeVerts[1], cubeVerts[3],  //back  
	cubeVerts[5], cubeVerts[3], cubeVerts[7],
	cubeVerts[4], cubeVerts[5], cubeVerts[7],  //left  
	cubeVerts[4], cubeVerts[7], cubeVerts[6],
	cubeVerts[4], cubeVerts[0], cubeVerts[1],  //top 
	cubeVerts[4], cubeVerts[1], cubeVerts[5],
	cubeVerts[6], cubeVerts[7], cubeVerts[3],  //bottom 
	cubeVerts[6], cubeVerts[3], cubeVerts[2],
	
}; 


vec4 right =	vec4( 1.0f, 0.0f, 0.0f, 0.0f);
vec4 left =		vec4(-1.0f, 0.0f, 0.0f, 0.0f);
vec4 top =		vec4( 0.0f, 1.0f, 0.0f, 0.0f);
vec4 bottom =	vec4( 0.0f,-1.0f, 0.0f, 0.0f);
vec4 front =	vec4( 0.0f, 0.0f, 1.0f, 0.0f);
vec4 back =		vec4( 0.0f, 0.0f,-1.0f, 0.0f);

vec4 white =	vec4( 1.0f, 1.0f, 1.0f, 1.0f);

vec4 normals[] = 
{
	//Normals for Wire Cube
	front, front, front, front, front,
	right, right, right, right, right,
	back, back, back, back, back, 
	left, left, left, left, left,
	top, top, top, top, top,
	bottom, bottom, bottom, bottom, bottom,
	
	//Normals for Solid Cube
	front, front, front, front, front, front,
	right, right, right, right, right, right,
	back, back, back, back, back, back,
	left, left, left, left, left, left,
	top, top, top, top, top, top,
	bottom, bottom, bottom, bottom, bottom, bottom
	
};


//Vertex array objects
GLuint cubes, torus;

//Shader Programs
GLuint program1, program2;

//Transformation Uniforms for Shader
GLint mvIndex;
GLint pIndex;

//Transformation Matrices
mat4 p, mv;

//Lighting Uniforms for Shader
GLint iLightPosition;
GLint ambientProduct;
GLint diffuseProduct;
GLint specularProduct;
GLint emission;
GLint shininess;
GLint lighting;
GLint uColor;

//Initial or "Default" Lighting Values
vec4 LightPosition(0.0f,0.0f,10.0f,0.0f);
vec4 LightAmbient(0.2f,0.2f,0.2f,1.0f);
vec4 LightDiffuse(1.0f,1.0f,1.0f,1.0f);
vec4 LightSpecular(1.0f,1.0f,1.0f,1.0f);

//Initial or "Default" Material Properties
vec4 MatAmbient(0.2f,0.2f,0.2f,1.0f);
vec4 MatDiffuse(0.8f,0.8f,0.8f,1.0f);
vec4 MatSpecular(0.0f,0.0f,0.0f,1.0f);
vec4 MatEmission(0.0f,0.0f,0.0f,1.0f);
GLfloat MatShininess = 5.0f;

//Number of vertices in Torus
GLuint ntVerts;

//----------------------------------------------------------------------------

// OpenGL initialization
void init()
{	
	//Set up shaders and VAOs
	program1 = InitShader( "vShaderL4.glsl", "fShaderL4.glsl" );

	glUseProgram( program1 );
	glGenVertexArrays( 1, &cubes );
	glBindVertexArray( cubes );


	GLuint buffer;
	glGenBuffers( 1, &buffer );
	glBindBuffer( GL_ARRAY_BUFFER, buffer );
	glBufferData( GL_ARRAY_BUFFER, sizeof(points)+sizeof(normals),
		NULL, GL_STATIC_DRAW );
	glBufferSubData( GL_ARRAY_BUFFER, 0, sizeof(points), points );
	glBufferSubData( GL_ARRAY_BUFFER, sizeof(points), sizeof(normals), normals );
	GLuint vPosition = glGetAttribLocation( program1, "vPosition" );
	glEnableVertexAttribArray( vPosition );
	glVertexAttribPointer( vPosition, 4, GL_FLOAT, GL_FALSE, 0,
		BUFFER_OFFSET(0) );
	GLuint vNormal = glGetAttribLocation( program1, "vNormal" ); 
	glEnableVertexAttribArray( vNormal );

	glVertexAttribPointer( vNormal, 4, GL_FLOAT, GL_FALSE, 0,
		BUFFER_OFFSET(sizeof(points)) );


	//Get locations of transformation matrices from shader
 	mvIndex = glGetUniformLocation(program1, "mv");
	pIndex = glGetUniformLocation(program1, "p");

	//Get locations of lighting uniforms from shader
	iLightPosition = glGetUniformLocation(program1, "lightPosition");
	ambientProduct = glGetUniformLocation(program1, "ambientProduct");
	diffuseProduct = glGetUniformLocation(program1, "diffuseProduct");
	specularProduct = glGetUniformLocation(program1, "specularProduct");
	emission = glGetUniformLocation(program1, "emission");
	shininess = glGetUniformLocation(program1, "shininess");
	lighting = glGetUniformLocation(program1, "lighting");
	uColor = glGetUniformLocation(program1, "uColor");

	//Set default lighting and material properties in shader.
	glUniform4fv(iLightPosition, 1, LightPosition);
	glUniform4fv(ambientProduct, 1, MatAmbient*LightAmbient);
	glUniform4fv(diffuseProduct, 1, MatDiffuse*LightDiffuse);
	glUniform4fv(specularProduct, 1, MatSpecular*LightSpecular);
	glUniform4fv(emission, 1, MatEmission);
	glUniform1f(shininess, MatShininess);
	glUniform1f(lighting, GL_TRUE);
	glUniform4f(uColor, 1.0f, 1.0f, 1.0f, 1.0f);

    urgl.connectShader(program1,"vPosition", "vNormal", "none");
    glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------

void display( void )
{
    //Clear the drawing buffer
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    
    //Reset ModelView Matrix
    mv = mat4();
    
    //set up viewing transformation
    point4 eye(0,0,10,1);
    point4 at(0,0,0,1);
    vec4   up(0,1,0,0);
    mv *= LookAt(eye, at, up);
    
    //Object rotation variables
    static GLfloat ry = 0, rx = 0;
    ry += 0.8f;
    rx += 2.0f;
    
    //Torus Transform
    mat4 tor = mv* RotateY(-ry);
    glUniformMatrix4fv(mvIndex, 1, GL_TRUE, tor);
    
    //Draw Torus
    urgl.drawSolidTorus(.5, 2, 10, 20);
    
    //Cube Transform
    mat4 cube = mv * RotateY(ry) * RotateX(rx);
    glUniformMatrix4fv(mvIndex, 1, GL_TRUE, cube);
    
    //Draw Cube
    glBindVertexArray(cubes);
    glDrawArrays(GL_TRIANGLES, solidCubeStart, solidCubeVertices);
    
    //Show what was drawn
    glutSwapBuffers();
    
    
}

//----------------------------------------------------------------------------

void reshape (int w, int h)
{
	GLfloat aspect = GLfloat(w)/GLfloat(h);
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	p = mat4();
	p *= Perspective(50.0f, aspect,  0.5f, 20.0f);
	glUniformMatrix4fv(pIndex, 1, GL_TRUE, p);
}


void keyboard( unsigned char key, int x, int y )
{
	switch( key ) {

	case 033:  // Escape key
	case 'q': case 'Q':
		exit( EXIT_SUCCESS );
		break;
	}
}

//----------------------------------------------------------------------------

void timer(int i)
{
	glutPostRedisplay();
	glutTimerFunc(33, timer, 1);
}

//----------------------------------------------------------------------------

int main( int argc, char **argv )
{
	glutInit( &argc, argv );
	glutInitWindowSize( 512, 512 );

#ifdef __APPLE__
	glutInitDisplayMode( GLUT_3_2_CORE_PROFILE | GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
#else
	// If you are using freeglut, the next two lines will check if 
	// the code is truly 3.2. Otherwise, comment them out
	glutInitContextVersion( 3, 2 );
	glutInitContextProfile( GLUT_CORE_PROFILE );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
#endif

	glutCreateWindow( "Lab 4 - Lighting Demo" );
	glewExperimental = GL_TRUE;
	glewInit();

	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutTimerFunc(17, timer, 1);;
	glutMainLoop();
	return 0;
}
